/*------------------------------------------------------------------------------
 * $AstecId: uxbr_menus.c,v 1.3 1998/05/11 05:19:31 yamato Exp $
 * $Date: 1996/11/09 16:38:26 $			$Revision: 2.4.65.1 $
 *------------------------------------------------------------------------------
 *  SCCSID -- @(#)uxbr_menus.c	1.1    96/04/10 KL Group Inc.
 *------------------------------------------------------------------------------
 *		Copyright (c) 1991, Visual Edge Software Ltd.
 *
 * ALL  RIGHTS  RESERVED.  Permission  to  use,  copy,  modify,  and
 * distribute  this  software  and its documentation for any purpose
 * and  without  fee  is  hereby  granted,  provided  that the above
 * copyright  notice  appear  in  all  copies  and  that  both  that
 * copyright  notice and this permission notice appear in supporting
 * documentation,  and that  the name of Visual Edge Software not be
 * used  in advertising  or publicity  pertaining to distribution of
 * the software without specific, written prior permission. The year
 * included in the notice is the year of the creation of the work.
 *------------------------------------------------------------------------------
 * This file contains stub functions you can fill in to extend the
 * interface browser menu system.
 *----------------------------------------------------------------------------*/

/*--- include files ---*/

#include <UxLib.h>
#include <UxRowCol.h>
#include <UxTogB.h>
#include <UxTogBG.h>
#include <UxLabel.h>
#include <UxLabelG.h>
#include <UxPushB.h>
#include <UxPushBG.h>
#include <UxSep.h>
#include <UxSepG.h>
#include <UxCascB.h>
#include <UxCascBG.h>

#include "cat_macros.h"


#if	defined(XRT) && defined(HASXRTSNAPSHOT)
extern void XrtWidgetSnapshotCBProc();
#endif	/* defined(XRT) && defined(HASXRTSNAPSHOT) */

/*--- forward declaration of static functions ---*/

static void FreeContext();

/*
*** This type can be filled in with swidget fields for your menu entries.
*/

typedef struct 
{
#if	defined(XRT) && defined(HASXRTSNAPSHOT)
  swidget xrtSnapshot;
#else	/* defined(XRT) && defined(HASXRTSNAPSHOT) */
  int mumble;
#endif	/* defined(XRT) && defined(HASXRTSNAPSHOT) */
} Context_t;

/*
*** New cascadeButtons and subMenus may be added to browser's menu bar.
*/

Context_t	*
UxAddToBrMenuBar( brMenuBar )
   swidget brMenuBar;
{
   Context_t *context = NULL;

   /*
   *** Steps to follow:
   ***
   *** 1.) Declare your extra swidgets as fields in the Context_t structure; 
   ***     this includes menus for subMenuId.  You will used an instance
   ***	   of this structure to store the extra swidgets for each browser;
   ***	   there may be several browsers active at once.
   ***
   *** 2.) In this function, allocate a context instance for the new swidgets.
   ***	   Create any new cascades+submenus for the menu bar,
   ***	   assigning them to the context fields.   Entries you
   ***	   add to the existing pulldowns will be built in the functions below,
   ***	   and stored in the same context structure.
   ***
   *** 3.) See sample code below:
   ***
   ***        swidget     cascade, menu, button1, button2;
   ***
   ***        menu = UxCreateRowColumn( "menu", brMenuBar );
   ***        UxPutRowColumnType( menu, "menu_pulldown" );
   ***        cascade = UxCreateCascadeButton( "Cascade", brMenuBar );
   ***        UxPutMnemonic( cascade, "C" );
   ***        button1 = UxCreatePushButtonGadget( "Button1", menu );
   ***        button2 = UxCreatePushButtonGadget( "Button2", menu );
   ***
   ***        UxCreateWidget( menu );
   ***        UxPutSubMenuId( cascade, "menu" );
   ***        UxCreateWidget( cascade);
   ***        UxCreateWidget( button1 );
   ***        UxCreateWidget( button2 );
   ***
   ***        UxAddCallback (button1, XmNactivateCallback, MyCallback1, data);
   ***        UxAddCallback (button2, XmNactivateCallback, MyCallback2, data);
   ***
   ***        context = ( Context_t * )malloc( sizeof( Context_t ) );
   ***	      context->cascade = cascade;
   ***	      context->menu = menu;
   ***	      context->button1 = button1;
   ***	      context->button2 = button2;
   ***
   */
   
   context = (Context_t *)malloc(sizeof(Context_t));
   UxAddCallback( brMenuBar, XmNdestroyCallback, FreeContext, context );
   return context;
}


/*
*** New menu items may be added to the browser's pulldown menus.
*** The 1st parameter is the menu to be extended.
*** The 2nd parameter is the context returned by UxAddToBrMenuBar().
*/

void UxAddToBrFileMenu( brFileMenu, context )
   swidget 	brFileMenu;
   Context_t    *context;
{
#if	defined(XRT) && defined(HASXRTSNAPSHOT)
	swidget xrtSnapshot;

	xrtSnapshot = UxCreatePushButton( "xrtSnapshot", brFileMenu );
	UxPutLabelString ( xrtSnapshot, CGETS_KL(PRINT_WIDGET_SNAPSHOT) );
	UxPutMnemonic(xrtSnapshot, CGETS_KL(PRINT_WIDGET_SNAPSHOT_M));
	UxPutCreateManaged( xrtSnapshot, "true" );
	UxCreateWidget( xrtSnapshot );
	UxAddCallback( xrtSnapshot, XmNactivateCallback, XrtWidgetSnapshotCBProc, (XtPointer) NULL );
	
	context->xrtSnapshot = xrtSnapshot; 
#endif	/* defined(XRT) && defined(HASXRTSNAPSHOT) */
}

void UxAddToBrCreateMenu( brCreateMenu, context )
   swidget 	brCreateMenu;
   Context_t    *context;
{
}

void UxAddToBrEditMenu( brEditMenu, context )
   swidget 	brEditMenu;
   Context_t    *context;
{
}

void UxAddToBrViewMenu( brViewMenu, context )
   swidget 	brViewMenu;
   Context_t    *context;
{
}

void UxAddToBrOptionsMenu( brOptionsMenu, context )
   swidget      brOptionsMenu;
   Context_t    *context;
{
}

void UxAddToBrModeMenu( brModeMenu, context )
   swidget      brModeMenu;
   Context_t    *context;
{
}

void UxAddToBrToolsMenu( brToolsMenu, context )
   swidget      brToolsMenu;
   Context_t    *context;
{
}

void UxAddToBrHelpMenu( brMenu, context )
   swidget      brMenu;
   Context_t    *context;
{
}


/*
*** The sensitivities of the items added to the browser's pulldowns above
*** can be controlled from the following functions, which are called
*** whenever the menus are cascaded.  
***	topSwidgetOfIface   refers to the interface in the browser.
***     context             is the return value of UxAddToBrMenuBar().
*/

void UxBrFileCascCBSup( topSwidgetOfIface, context )
   swidget 	topSwidgetOfIface;
   Context_t    *context;
{
#if	defined(XRT) && defined(HASXRTSNAPSHOT)
  
	char* on;

	if ( UxIsValidSwidget( topSwidgetOfIface ) )
		on = "true";
	else
		on = "false";

	UxPutSensitive( context->xrtSnapshot, on);

#endif	/* defined(XRT) && defined(HASXRTSNAPSHOT) */
}

void UxBrCreateCascCBSup( topSwidgetOfIface, context )
   swidget 	topSwidgetOfIface;
   Context_t    *context;
{
}

void UxBrEditCascCBSup( topSwidgetOfIface, context )
   swidget      topSwidgetOfIface;
   Context_t    *context;
{
}

void UxBrViewCascCBSup( topSwidgetOfIface, context )
   swidget      topSwidgetOfIface;
   Context_t    *context;
{
}

void UxBrOptionsCascCBSup( topSwidgetOfIface, context )
   swidget      topSwidgetOfIface;
   Context_t    *context;
{
}

void UxBrToolsCascCBSup( topSwidgetOfIface, context )
   swidget      topSwidgetOfIface;
   Context_t    *context;
{
}

void UxBrHelpCascCBSup( topSwidgetOfIface, context )
   swidget      topSwidgetOfIface;
   Context_t    *context;
{
}

/*
*** Sample destroy callback for brMenuBar
*/

static void FreeContext( wid, clientData, callData )
   Widget wid;
   char   *clientData,
          *callData;
{
   if( clientData ) free( clientData );
}
