/*******************************************************************************
  $AstecId: seriesEditor.c,v 1.10 2006/08/30 01:16:16 kosaka Exp $
       seriesEditor.c
       (Generated from interface file seriesEditor.i)
       Associated Header file: seriesEditor.h
*******************************************************************************/

#ifdef	HASXRTTABLE

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <Xm/RepType.h>
#include "UxLib.h"
#include "UxArrB.h"
#include "UxXrtTable.h"
#include "UxForm.h"
#include "UxTemplD.h"
#include <pedit-pop.h>
#include <uxgui.h>
#include <util.h>
#include <Xm/SelectioB.h>
#include <Xm/XrtTable.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include "cat_macros.h"


static String helpMsg[] = {
"XRT/table Series Editor",
"------------------------",
"",
"It contains a 3-column table, in which:",
"- the first and second columns allow you to set row/column elements of",
"  series resource value",
"- the third column allows to set the actual value of the resource",
"",
"The series editor implements a popup menu, with a list of the valid",
"entries for the current cell.  Use MB3 to activate the popup menu.",
"",
"Popup menu items:",
"- The first and second column of seriesEditor create a popup menu with",
"  a list of predefined entries for row/column elements of the series",
"  resource.",
"- A popup menu for the third column is created only for those series",
"  resources which have a predefined set of valid entries (all enum types",
"  + Boolean).  The entry list depends on the type of the series resource,",
"  and is unique for each series type.",
"",
"In addition, there are specialized resource editors available from",
"within SeriesEditor.  They are used to edit series resources whose type",
"is a color, font or pixmap. To popup these editors press '...' button.",
NULL
};

static void seAddMenuAction();
static void sePopupMenuAction();
static void seCreateClones();
static void seGetSeriesTokens();
static void sePutSeriesTokens();
static void seReleasePeSlot();
static void activateCloneCB();
static void popupMenuCB();
static void popdownMenuCB();

extern Boolean xrttbl_GetToken();

#define IS_NULL(s)    (((s) && !(s)[0]) || !(s))
#define XTFREE(ptr)	  XtFree((char *)(ptr))
#define XTREALLOC(ptr, type, size) \
((size) > 0 ? ptr = (type *)XtRealloc((char *)(ptr), (size)*sizeof(type)) \
: (XTFREE(ptr), ptr = (type *)NULL))

#define ROW_VALUE     0
#define COLUMN_VALUE  1
#define SERIES_VALUE  2
#define CLONED_COLUMN 3

static	int _UxIfClassId;

/*******************************************************************************
       The following header file defines the context structure.
*******************************************************************************/

#ifndef XKLOADDS
#define XKLOADDS
#endif /* XKLOADDS */

#define CONTEXT_MACRO_ACCESS 1
#include "seriesEditor.h"
#undef CONTEXT_MACRO_ACCESS

/*******************************************************************************
Auxiliary code from the Declarations Editor:
*******************************************************************************/

/*
 *  Generic XRT/table series resource editor
 */

void sePopupEditor( pedit, resname, popupFun, pickList)
    swidget  pedit;
    char     *resname;
    swidget  (* popupFun)();
    char **pickList;
{
    char *value, *raw_value;
    swidget tmpsw;
	
    /* Create the interface and save context information */
    tmpsw = create_seriesEditor(pedit);
    XrtSeriesEd = tmpsw;
    XrtPedit = pedit;
    SeCustomPopupFun = popupFun;
    SePickList = pickList;
    SeReservedRow = 0;
    SeCustomPopupIface = NULL;

    /* Create column of cloned pb only for series which require popup editors */
    if ( SeCustomPopupFun )
    {
	seCreateClones(UxGetWidget(XrtTableSeriesEd));	  
    }  

    /* Add popup menus */
    seAddMenuAction(UxGetWidget(XrtTableSeriesEd));
	
    /* Reserve PE slot while we are editing series */
    UxGUIPeReserve( XrtPedit, resname, XrtSeriesEd, &value, &XrtSeriesEdSlot);

    /* If the slot has a value, unqoute it and put it in seriesEditor */
    if (value)
    {
	raw_value = UxUnquoteWrap(value);
	seGetSeriesTokens(raw_value);
    }

    /* Adjust Editor Title */
    UxPutDialogTitle( XrtSeriesEd, resname);
	
    /* Display SeriesEditor */
    UxPopupInterface( XrtSeriesEd, no_grab);
}

/*
 * Release the current slot and destroy the interface
 */

static void seReleasePeSlot()
{
    UxGUIPeRelease( XrtPedit, XrtSeriesEd);
    UxDestroyInterface( XrtSeriesEd);
}

/*
 *  Deposit a new quoted string in the current slot
 */

static void seDepositInPe( series)
    String series;
{

    /* 
     *  If "series" is NULL ( SeriesEditor table empty) then force
     *  to deposit "" in PE 
     */

    if (!series)
    {
	series = "";
    }
    UxGUIPeDepositValue( XrtPedit, XrtSeriesEdSlot, UxQuoteWrap(series));
}


/*
 * Create column of cloned pb's,
 * but only for those which have additional popup editors (color,font,pixmap)
 */

static void seCreateClones(table)
	Widget table;
{
	Widget clonedPB;

	static XrtTblRange span = {-1, 2, -1, 3};
	static XrtTblRange *spans[] = {&span, NULL};

	clonedPB = XtVaCreateWidget("...",
			  xmPushButtonWidgetClass,	table, NULL);
	XtAddCallback(clonedPB, XmNactivateCallback, activateCloneCB, NULL);

	XtVaSetValues(table, XmNxrtTblNumColumns, 4, XmNxrtTblSpanList, spans,
	  	      XmNxrtTblContext, 
	  	      XrtTblSetContext(XRTTBL_ALLCELLS, CLONED_COLUMN),
		      XmNxrtTblWidgetContext, clonedPB, NULL);	
}

/*
 * This action routine creates popup menu which displays list of valid entries
 * for that cell.
 * Values are different for each column:
 * - 1st and 2nd column of seriesEditor have the same list of valid entries for
 *   table row/column.
 * - Entry list for 3rd column is created only for those series resources
 *   which have a predefined set of valid entries (all enum types + Boolean).
 *   Entry list depends of the type of series resource and it is unique
 *   for each type.
 */

static void
sePopupMenuAction(text, event, params, nparams)
Widget text;
XEvent *event;
String *params;
Cardinal *nparams;
{
  static char *rowcol_menu[] = {"ALL", "LABEL", "ALLCELLS", NULL};

  char **popup_menu = rowcol_menu;

	_UxCseriesEditor        *UxSaveCtx, *UxContext;

	/*	Widget items[XtNumber(popup_menu)];*/
	
	Widget pb, menu, shell;
	Widget table = XtParent(text);
	swidget sw_table = UxWidgetToSwidget( table );
	int row, column;
	int i;	

	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext = UxContext =
	  (_UxCseriesEditor *) UxGetContext( sw_table );

	if (!XrtTblEventToRowCol(table, event, &row, &column))
		return;

	/* Ignore label clicks */
	if (row == XRTTBL_LABEL || column == XRTTBL_LABEL)
		return;

	/*
	 * If this is SERIES_VALUE column then set different popup_menu
	 */
	   
	if (column == SERIES_VALUE) {
	  if ( SePickList ) 
		popup_menu = SePickList;
	  else 
		return;
	}
	  
	/* Pop up menu */
	menu = XmCreatePopupMenu(text, "seriesMenu", NULL, 0);
	for (i=0; popup_menu[i]; i++) {
	  pb = XtVaCreateManagedWidget("pb", xmPushButtonWidgetClass, menu,
					XtVaTypedArg,	XmNlabelString,
					XmRString,	popup_menu[i], 
					strlen(popup_menu[i])+1, NULL);
	  XtAddCallback(pb, XmNactivateCallback, popupMenuCB, table);
	};

	/* Install popdown callback on shell parent of menu */
	shell = menu;
	while (shell && !XtIsShell(shell)) {
	  shell = XtParent(shell);
	}
	XtAddCallback(shell, XmNpopdownCallback, popdownMenuCB, NULL);
	
	XmMenuPosition(menu, (XButtonPressedEvent *)event);
	XtManageChild(menu);

	UxSeriesEditorContext = UxSaveCtx;
}

static void seAddMenuAction( table)
Widget table;
{

	static XtActionsRec	actions[] = {
		{"sePopupMenuAction",	   (XtActionProc) sePopupMenuAction }
	};
	static char trans[] = "#override\n\
<Btn3Down>: sePopupMenuAction()";
	
	  Widget  text;

	  XtVaGetValues(table,
					XmNxrtTblText,	&text, NULL);
	  XtAppAddActions(XtWidgetToApplicationContext(table),
					  actions,XtNumber(actions));
	  XtOverrideTranslations(text, XtParseTranslationTable(trans));
}

/*
 * This is converter from "()" list of series to seriesEditor value
 * Get next whitespace-delimited token ("token" must be freed after use)
 */

static Boolean GetSpaceToken(string, token)
String *string;
String *token;
{
	String tmp;

	*token = NULL;
	if (!*string || **string == '\0' || strlen(*string) == 0)
		return (False);

	/* Strip leading whitespace */
	for (; **string != '\0' && isspace(**string); (*string)++) 
		NULL;

	if (**string == '\0')
		return (False);

	*token = XtMalloc(strlen(*string) + 1);
	tmp = *token;

	while ((**string) != '\0') {
		if (isspace(**string)) {
			*tmp = '\0';
			(*string)++;
			return (True);
		}
		else {
			*tmp = **string;
			tmp++; 
			(*string)++;
		}
	}

    *tmp = '\0';
    return (True);
}

/*
 * Take value form PE (string) and put it in SeriesEditor table
 */

static void
seGetSeriesTokens(string)
String string;
{

	Boolean substring_found, converted = True;
	String p, substring,  start, start_copy;

	typedef struct {
	  String  row;
	  String  column;
	  String  seriesValue;
	} seriesTable;
	
	int i = 0;
	seriesTable *list = NULL;
	int length = 0;

	Widget table = UxGetWidget(XrtTableSeriesEd);
    
	start = start_copy = XtNewString(string);
	if (IS_NULL(start)) {
	  return;
	}  

	/*
	 * Find substring between ()
	 */
	substring_found = xrttbl_GetToken(&start, ')', &substring);
	while (substring_found) {

		/* Find first '(' */
		if (!(converted = xrttbl_GetToken(&substring, '(', &p))) {
			break;
		}
		XtFree(p);

		XTREALLOC(list, seriesTable, length+1); 
		list[length].seriesValue = NULL;

		if (!(converted = GetSpaceToken(&substring, &list[length].row)))
			break;
			
		if (!(converted = GetSpaceToken(&substring, &list[length].column)))
			break;
		
		if (!(converted = xrttbl_GetToken(&substring, ')',
										  &list[length].seriesValue)))
			break;
		
		length++;
		substring_found = xrttbl_GetToken(&start, ')', &substring);
	}	

    if (!converted) {
		if (length == 0) {
			XTFREE(list); /* rb XTFREE */
			list = NULL;
			return;
		} else if (list) {
			list[length].seriesValue = NULL;
		    fprintf( stderr, "Series Editor: Parsing error\n");
		}	
	} else {
		XTREALLOC(list, seriesTable, length+1); 
		list[length].seriesValue = NULL;
	}	

	for( i=0; i<= 50 && list[i].seriesValue; i++) {

	  XtVaSetValues( table,
			 XmNxrtTblContext, XrtTblSetContext(i, ROW_VALUE),
			 XmNxrtTblCellValueContext, XmStringCreateLocalized(list[i].row),
			 NULL);
	  XtVaSetValues( table,
			 XmNxrtTblContext, XrtTblSetContext(i, COLUMN_VALUE),
			 XmNxrtTblCellValueContext, XmStringCreateLocalized(list[i].column),
			 NULL);
	  XtVaSetValues( table,
			 XmNxrtTblContext, XrtTblSetContext(i, SERIES_VALUE),
			 XmNxrtTblCellValueContext, XmStringCreateLocalized(list[i].seriesValue),
			 NULL);
	}
	if (!list)
		return;
	for (i=0; list[i].seriesValue; i++) {
		XtFree(list[i].seriesValue);
		XtFree(list[i].row);
		XtFree(list[i].column);
	}
	XTFREE(list); /* rb XTFREE */
}


/*
 * Concatenate and put value form SeriesEditor table into reserved slot in PE
 */
static void sePutSeriesTokens()
{
  XrtTblCellValues *cells;
  int i, j, rows, columns;

  int length = 0;
  String seriesRow, seriesColumn, seriesValue,
	     resource = NULL,
	     token = NULL;
  Boolean start = TRUE;

  UxPutXrtTblXmStringGetMode( XrtTableSeriesEd, "Get_String"),
  XrtTblCommitEdit( UxGetWidget(XrtTableSeriesEd), False);
  XtVaGetValues( UxGetWidget(XrtTableSeriesEd), XmNxrtTblCellValues, &cells, NULL);
  XrtTblGetCellValuesSize(cells, &rows, &columns, NULL);

  for(i=0; i<= rows; i++) {
	if(IS_NULL( seriesRow    = (char *) CELL_VALUE(cells, i, ROW_VALUE ))) continue;
	if(IS_NULL( seriesColumn = (char *) CELL_VALUE(cells, i, COLUMN_VALUE))) continue;
	if(IS_NULL( seriesValue  = (char *) CELL_VALUE(cells, i, SERIES_VALUE ))) continue;

	token = XtMalloc(strlen(seriesRow) + strlen(seriesColumn) + strlen(seriesValue) +  strlen(" (  )") + 1);
	if(start) {
	  start = FALSE;
	  sprintf(token, "(%s %s %s)",seriesRow, seriesColumn, seriesValue);
	} else
	  sprintf(token, " (%s %s %s)",seriesRow, seriesColumn, seriesValue);

	if( resource == NULL )
	  resource = XtCalloc(1, length + strlen(token)+1);
	else
	  resource = XtRealloc(resource, length + strlen(token)+1);
	
	strcat(resource, token);
	if(resource)
	  length = strlen(resource);
	
	XtFree(token);
  }
  seDepositInPe( resource);
  seReleasePeSlot();
  XtFree(resource);
}

void seDepositSeriesValue(se, value)
swidget se;
char *value;
{
	_UxCseriesEditor        *UxSaveCtx, *UxContext;

	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext =	(_UxCseriesEditor *) UxGetContext( se );

	if ( value ) 
	{
		XtVaSetValues( UxGetWidget(XrtTableSeriesEd),
			XmNxrtTblContext, 
			XrtTblSetContext( SeReservedRow, SERIES_VALUE),
			XmNxrtTblCellValueContext, XmStringCreateLocalized(value),
			NULL);
	}

	UxSeriesEditorContext = UxSaveCtx;
}

char* seReserveSeriesValue(se)
swidget se;
{
	_UxCseriesEditor        *UxSaveCtx, *UxContext;
        String initialValue;
        XmString value;


	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext =	(_UxCseriesEditor *) UxGetContext( se );

	XtVaSetValues(UxGetWidget(XrtTableSeriesEd), XmNxrtTblContext, 
		      XrtTblSetContext(SeReservedRow, SERIES_VALUE), NULL);

	/* assumes that XmNxrtTblXmStringgetMode is "Get_String" */
	XtVaGetValues(UxGetWidget(XrtTableSeriesEd), 
		      XmNxrtTblCellValueContext, &initialValue, NULL);

	UxSeriesEditorContext = UxSaveCtx;

	return UxQuoteWrap(initialValue);
}

void seReleaseSeriesValue(se)
swidget se;
{
	_UxCseriesEditor        *UxSaveCtx, *UxContext;

	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext =	(_UxCseriesEditor *) UxGetContext( se );

	SeCustomPopupIface = NULL;

	UxSeriesEditorContext = UxSaveCtx;
}

/*
 * These editors use pickList to allow editiong of series resources
 */

void
UxPopupXrtTblAlignmentSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"ALIGNMENT_TOPBEGIN",
	"ALIGNMENT_TOPCENTER",
	"ALIGNMENT_TOPEND",
	"ALIGNMENT_MIDDLEBEGIN",
	"ALIGNMENT_MIDDLECENTER",
	"ALIGNMENT_MIDDLEEND",
	"ALIGNMENT_BOTTOMBEGIN",
	"ALIGNMENT_BOTTOMCENTER",
	"ALIGNMENT_BOTTOMEND",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblBooleanSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"TRUE",
	"FALSE",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblBorderSideSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"BORDERSIDE_NONE",
	"BORDERSIDE_LEFT",
	"BORDERSIDE_TOP",
	"BORDERSIDE_BOTTOM",
	"BORDERSIDE_RIGHT",
	"BORDERSIDE_ALL",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblBorderTypeSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"BORDER_NONE",
	"BORDER_ETCHED_IN",
	"BORDER_ETCHED_OUT",
	"BORDER_IN",
	"BORDER_OUT",
	"BORDER_PLAIN",
	"BORDER_FRAME_IN",
	"BORDER_FRAME_OUT",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblCaseSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"CASE_AS_IS",
	"CASE_LOWER",
	"CASE_UPPER",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblDataTypeSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"TYPE_FIELD_CURRENCY",
	"TYPE_FIELD_DATE",
	"TYPE_FIELD_FLOAT",
	"TYPE_FIELD_INT",
	"TYPE_FIELD_BOOLEAN",
	"TYPE_FIELD_STRING",
	"TYPE_BOOLEAN",
	"TYPE_DOUBLE",
	"TYPE_FLOAT",
	"TYPE_INT",
	"TYPE_STRING",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblOverflowSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"OVERFLOW_NONE",
	"OVERFLOW_HORIZONTAL",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}

void
UxPopupXrtTblPixmapLayoutSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  static char *pickList[] = {
	"PIXMAP_TOPLEFT",
	"PIXMAP_TOPCENTER",
	"PIXMAP_TOPRIGHT",
	"PIXMAP_MIDDLELEFT",
	"PIXMAP_MIDDLECENTER",
	"PIXMAP_MIDDLERIGHT",
	"PIXMAP_BOTTOMLEFT",
	"PIXMAP_BOTTOMCENTER",
	"PIXMAP_BOTTOMRIGHT",
	"PIXMAP_TILE",
	NULL
  };

  sePopupEditor( pedit, resname, NULL, pickList);
}


void
UxPopupXrtTblSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
  sePopupEditor( pedit, resname, NULL, NULL);
}

/*
 * These editors spawn separate popups to edit values of series resources
 */

static swidget sePopupColorView()
{
	return UxGUIPopupBEColorView(XrtSeriesEd,
                                        "Value", NULL, NULL, 0, XrtSeriesEd,
                                        seReserveSeriesValue,
                                        seReleaseSeriesValue,
                                        seDepositSeriesValue);
}

static swidget sePopupFontView()
{
        return UxGUIPopupBEFontView(XrtSeriesEd,
                                        "Value", NULL, XrtSeriesEd,
                                        seReserveSeriesValue,
                                        seReleaseSeriesValue,
                                        seDepositSeriesValue);
}

static swidget sePopupPixView()
{
	return UxGUIPopupBEPixView(XrtSeriesEd,
                                        "Value", NULL, XrtSeriesEd,
                                        seReserveSeriesValue,
                                        seReleaseSeriesValue,
                                        seDepositSeriesValue);
}

void
UxPopupXrtTblColorSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
	  sePopupEditor( pedit,
					 resname,
					 sePopupColorView,
					 NULL);
}

void
UxPopupXrtTblFontSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
    sePopupEditor( pedit,
				   resname,
				   sePopupFontView,
				   NULL);
}

void
UxPopupXrtTblPixmapSeriesEditor( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{
    sePopupEditor( pedit,
				   resname,
				   sePopupPixView,
				   NULL);
}


/*******************************************************************************
       The following are callback functions.
*******************************************************************************/

static void popupMenuCB(pb, table, cbs)
Widget pb;
XtPointer table, cbs;
{
  XmString xmValue;
  String value;
  int row, column;
  
  XtVaGetValues(pb,
				XmNlabelString, &xmValue,
				NULL);
  value = XrtTblCvtXmStringToString(xmValue);

  XrtTblGetCurrentCell(table, &row, &column);
  XtVaSetValues(table,
				XmNxrtTblContext, XrtTblSetContext(row, column),
				XmNxrtTblCellValueContext, XmStringCreateLocalized(value),
				NULL);
  /* destroy popup interface */
}

static void popdownMenuCB(menu, data, cbs)
Widget menu;
XtPointer data, cbs;
{
  while (menu && !XtIsShell(menu)) {
	  menu = XtParent(menu);
  }
  XtDestroyWidget(menu);
}


static void activateCloneCB(clone, client_data, cbs)
Widget clone;
XtPointer client_data, cbs;
{
    _UxCseriesEditor   	*UxSaveCtx, *UxContext;
	
	Widget table;
	int row, column;

	XrtTblGetRowColByWidget(clone, &table, &row, &column);

	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext = UxContext =
			(_UxCseriesEditor *) UxGetContext( UxWidgetToSwidget( table ));

	XrtTblCommitEdit( table, False);
	XrtTblTraverseToCell( table, row, SERIES_VALUE, True);
	SeReservedRow = row;
						  
    if ( SeCustomPopupFun )
    {
        if ( SeCustomPopupIface == NULL )
	{
	    SeCustomPopupIface = SeCustomPopupFun();
	}
	else
	{
	    UxRaiseInterface(SeCustomPopupIface);
	}
    }
	
    UxSeriesEditorContext = UxSaveCtx;
}

static void  helpCB_seriesEditor( UxWidget, UxClientData, UxCallbackArg )
	Widget		UxWidget;
	XtPointer	UxClientData, UxCallbackArg;

{
	_UxCseriesEditor        *UxSaveCtx, *UxContext;
	swidget			UxThisWidget;
	
	Widget table, field;
	int row,column;
	  
	UxThisWidget = UxWidgetToSwidget( UxWidget );
	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext = UxContext =
			(_UxCseriesEditor *) UxGetContext( UxThisWidget );
	{
#ifdef USEXRTTOOLS
	  MtShowGearAboutDialog(UxGetWidget(XrtSeriesEd), helpMsg);
#else
	printf("%s\n", CGETS_KL(SERIES_MSG1));
	printf("------------------------\n");
	printf("\n");
	printf("%s\n", CGETS_KL(SERIES_MSG2));
	printf("%s\n", CGETS_KL(SERIES_MSG3));
	printf("%s\n", CGETS_KL(SERIES_MSG4));
	printf("\n");
	printf("%s\n", CGETS_KL(SERIES_MSG5));
	printf("\n");
	printf("%s\n", CGETS_KL(SERIES_MSG6));
	printf("%s\n", CGETS_KL(SERIES_MSG7));
	printf("%s\n", CGETS_KL(SERIES_MSG8));
	printf("\n");
	printf("%s\n", CGETS_KL(SERIES_MSG9));
#endif 
	}
	UxSeriesEditorContext = UxSaveCtx;
}

static void  okCallback_seriesEditor( UxWidget, UxClientData, UxCallbackArg )
	Widget		UxWidget;
	XtPointer	UxClientData, UxCallbackArg;

{
	_UxCseriesEditor        *UxSaveCtx, *UxContext;
	swidget			UxThisWidget;

	UxThisWidget = UxWidgetToSwidget( UxWidget );
	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext = UxContext =
			(_UxCseriesEditor *) UxGetContext( UxThisWidget );

    sePutSeriesTokens();
	
	UxSeriesEditorContext = UxSaveCtx;
}

static void  cancelCB_seriesEditor( UxWidget, UxClientData, UxCallbackArg )
	Widget		UxWidget;
	XtPointer	UxClientData, UxCallbackArg;

{
	_UxCseriesEditor *UxSaveCtx, *UxContext;
	swidget			UxThisWidget;

	UxThisWidget = UxWidgetToSwidget( UxWidget );
	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext = UxContext =
			(_UxCseriesEditor *) UxGetContext( UxThisWidget );
	{
	  /*
	   *  Release currently reserved slot before throwing away interface
	   */
	  seReleasePeSlot();
	}
	UxSeriesEditorContext = UxSaveCtx;
}

static void  destroyCB_seriesEditor( UxWidget, UxClientData, UxCallbackArg )
	Widget		UxWidget;
	XtPointer	UxClientData, UxCallbackArg;

{
	_UxCseriesEditor *UxSaveCtx, *UxContext;
	swidget			UxThisWidget;

	UxThisWidget = UxWidgetToSwidget( UxWidget );
	UxSaveCtx = UxSeriesEditorContext;
	UxSeriesEditorContext = UxContext =
			(_UxCseriesEditor *) UxGetContext( UxThisWidget );
	{
	  if( SeCustomPopupIface) {
		UxDestroyInterface( SeCustomPopupIface);
	  }	
	}
	UxSeriesEditorContext = UxSaveCtx;
}



/*******************************************************************************
       The 'build_' function creates all the swidgets and X widgets,
       and sets their properties to the values specified in the
       Property Editor.
*******************************************************************************/

static swidget	_Uxbuild_seriesEditor()
{
	char label[128];
	/* Create the swidgets */


	/* Creation of seriesEditor */
	seriesEditor = UxCreateTemplateDialog( "seriesEditor", UxParent );
	UxPutContext( seriesEditor, UxSeriesEditorContext );
	UxPutClassCode( seriesEditor, _UxIfClassId );
	UxPutDefaultShell( seriesEditor, "topLevelShell" );

	UxPutMsgDialogType( seriesEditor, "dialog_template" ),
	UxPutX( seriesEditor, 480 ),
	UxPutY( seriesEditor, 360 ),
	UxPutUnitType( seriesEditor, "pixels" ),
	UxPutOkLabelString( seriesEditor, CGETS_COM(PB_APPLY) ),
	UxPutCancelLabelString( seriesEditor, CGETS_COM(PB_CANCEL) ),
	UxPutHelpLabelString( seriesEditor, CGETS_KL(PB_HELP) ),
	UxCreateWidget( seriesEditor );

	UxAddCallback( seriesEditor, XmNcancelCallback,
		(XtCallbackProc) cancelCB_seriesEditor,
		(XtPointer) UxSeriesEditorContext );
	UxAddCallback( seriesEditor, XmNokCallback,
		(XtCallbackProc) okCallback_seriesEditor,
		(XtPointer) UxSeriesEditorContext );
	UxAddCallback( seriesEditor, XmNhelpCallback,
		(XtCallbackProc) helpCB_seriesEditor,
		(XtPointer) UxSeriesEditorContext );

	UxAddCallback( seriesEditor, XmNdestroyCallback,
		(XtCallbackProc) destroyCB_seriesEditor,
		(XtPointer) UxSeriesEditorContext );

	/* Creation of form1 */
	form1 = UxCreateForm( "form1", seriesEditor );
	UxCreateWidget( form1 );

	/* Creation of XrtTableSeriesEd */
	XrtTableSeriesEd = UxCreateXrtTable( "XrtTableSeriesEd", form1 );
	UxPutX( XrtTableSeriesEd, 23 ),
	UxPutY( XrtTableSeriesEd, 15 ),
	UxPutXrtTblNumColumns( XrtTableSeriesEd, 3 ),
	UxPutXrtTblNumRows( XrtTableSeriesEd, 50 ),
	UxPutXrtTblCharHeightSeries( XrtTableSeriesEd, "(ALL ALL 1)" ),
	UxPutXrtTblCharWidthSeries( XrtTableSeriesEd, "(ALL ALL 8) (ALL 2 14) (ALL 3 2)" ),
	strcpy(label, CGETS_KL(ROW));	strcat(label, ",");
	strcat(label, CGETS_KL(COLUMN));strcat(label, ",");
	strcat(label, CGETS_KL(VALUE));
	UxPutXrtTblColumnLabels( XrtTableSeriesEd, label);
	UxPutXrtTblBorderTypeSeries( XrtTableSeriesEd, "(LABEL ALL BORDER_FRAME_OUT)" ),
	UxPutXrtTblJumpScroll( XrtTableSeriesEd, "Jump_All" ),
	UxPutXrtTblVisibleRows( XrtTableSeriesEd, 6 ),
	UxPutXrtTblAllowCellResize( XrtTableSeriesEd, "Resize_None" ),
	UxPutXrtTblAllowResize( XrtTableSeriesEd, "Resize_All" ),
	UxPutXrtTblFrameShadowThickness( XrtTableSeriesEd, 3 ),
	UxPutXrtTblFrameBorderType( XrtTableSeriesEd, "Border_Out" ), 
	UxPutXrtTblXmStringGetMode( XrtTableSeriesEd, "Get_String"), 
	UxPutBottomAttachment( XrtTableSeriesEd, "attach_self");
	UxPutLeftAttachment( XrtTableSeriesEd, "attach_self");
	UxPutRightAttachment( XrtTableSeriesEd, "attach_self");
	UxPutTopAttachment( XrtTableSeriesEd, "attach_self");
	UxCreateWidget( XrtTableSeriesEd );

	UxAddCallback( seriesEditor, XmNdestroyCallback,
				   (XtCallbackProc) UxDestroyContextCB,
				   (XtPointer) UxSeriesEditorContext);

	/* UxRealizeInterface creates the X windows for the widgets above. */
	UxRealizeInterface( seriesEditor );

	return ( seriesEditor );
}

/*******************************************************************************
       The following is the 'Interface function' which is the
       external entry point for creating this interface.
       This function should be called from your application or from
       a callback function.
*******************************************************************************/

swidget	create_seriesEditor( _UxUxParent )
	swidget	_UxUxParent;
{
	swidget                 rtrn;
	_UxCseriesEditor        *UxContext;
	static int		_Uxinit = 0;

	UxSeriesEditorContext = UxContext =
		(_UxCseriesEditor *) UxNewContext( sizeof(_UxCseriesEditor), False );

	UxParent = _UxUxParent;

	if ( ! _Uxinit )
	{
		_UxIfClassId = UxNewInterfaceClassId();
		_Uxinit = 1;
	}

	rtrn = _Uxbuild_seriesEditor();

	return(rtrn);
}

#endif	/* HASXRTTABLE */
